searchData={"items":[{"type":"module","title":"Styler","doc":"Styler is a formatter plugin with stronger opinions on code organization, multi-line defs and other code-style matters.","ref":"Styler.html"},{"type":"behaviour","title":"Styler.Style","doc":"A Style takes AST and returns a transformed version of that AST.\n\nBecause these transformations involve traversing trees (the \"T\" in \"AST\"), we wrap the AST in a structure\ncalled a Zipper to facilitate walking the trees.","ref":"Styler.Style.html"},{"type":"function","title":"Styler.Style.displace_comments/2","doc":"Set the line of all comments with `line` in `range_start..range_end` to instead have line `range_start`","ref":"Styler.Style.html#displace_comments/2"},{"type":"function","title":"Styler.Style.empty_map?/1","doc":"Returns true if the ast represents an empty map","ref":"Styler.Style.html#empty_map?/1"},{"type":"function","title":"Styler.Style.ensure_block_parent/1","doc":"Returns the current node (wrapped in a `__block__` if necessary) if it's a valid place to insert additional nodes","ref":"Styler.Style.html#ensure_block_parent/1"},{"type":"function","title":"Styler.Style.find_nearest_block/1","doc":"Returns a zipper focused on the nearest node where additional nodes can be inserted (a \"block\").\n\nThe nearest node is either the current node, an ancestor, or one of those two but wrapped in a new `:__block__` node.","ref":"Styler.Style.html#find_nearest_block/1"},{"type":"callback","title":"Styler.Style.run/2","doc":"`run` will be used with `Zipper.traverse_while/3`, meaning it will be executed on every node of the AST.\n\nYou can skip traversing parts of the tree by returning a Zipper that's further along in the traversal, for example\nby calling `Zipper.skip(zipper)` to skip an entire subtree you know is of no interest to your Style.","ref":"Styler.Style.html#c:run/2"},{"type":"function","title":"Styler.Style.set_line/3","doc":"Recursively sets `:line` meta to `line`. Deletes `:newlines` unless `delete_lines: false` is passed","ref":"Styler.Style.html#set_line/3"},{"type":"function","title":"Styler.Style.shift_comments/2","doc":"Perform a series of shifts in a single pass.\n\nWhen shifting comments from block A to block B, naively using two passes of `shift_comments/3` would result\nin all comments ending up in either region A or region B (because A would move to B, then all B back to A)\nThis function exists to make sure that a comment is only moved once during the swap.","ref":"Styler.Style.html#shift_comments/2"},{"type":"function","title":"Styler.Style.shift_comments/3","doc":"Change the `line` of all comments with `line` in `range` by adding `delta` to it.\nA positive delta will move the lines further down a file, while a negative delta will move them up.","ref":"Styler.Style.html#shift_comments/3"},{"type":"function","title":"Styler.Style.shift_line/2","doc":"Recursively updates `:line` meta by adding `delta`","ref":"Styler.Style.html#shift_line/2"},{"type":"function","title":"Styler.Style.update_all_meta/2","doc":"Traverses an ast node, updating all nodes' meta with `meta_fun`","ref":"Styler.Style.html#update_all_meta/2"},{"type":"type","title":"Styler.Style.context/0","doc":"","ref":"Styler.Style.html#t:context/0"},{"type":"module","title":"Styler.Style.Blocks","doc":"Simple 1-1 rewrites all crammed into one module to make for more efficient traversals\n\nCredo Rules addressed:\n\n* Credo.Check.Consistency.ParameterPatternMatching\n* Credo.Check.Readability.LargeNumbers\n* Credo.Check.Readability.ParenthesesOnZeroArityDefs\n* Credo.Check.Readability.PreferImplicitTry\n* Credo.Check.Readability.WithSingleClause\n* Credo.Check.Refactor.CaseTrivialMatches\n* Credo.Check.Refactor.CondStatements\n* Credo.Check.Refactor.RedundantWithClauseResult\n* Credo.Check.Refactor.WithClauses","ref":"Styler.Style.Blocks.html"},{"type":"function","title":"Styler.Style.Blocks.run/2","doc":"","ref":"Styler.Style.Blocks.html#run/2"},{"type":"module","title":"Styler.Style.Defs","doc":"Styles function heads so that they're as small as possible.\n\nThe goal is that a function head fits on a single line.\n\nThis isn't a Credo issue, and the formatter is fine with either approach. But Styler has opinions!\n\nEx:\n\nThis long declaration\n\n    def foo(%{\n      bar: baz\n    }) do\n      ...\n    end\n\nBecomes\n\n    def foo(%{bar: baz}) do\n      ...\n    end","ref":"Styler.Style.Defs.html"},{"type":"function","title":"Styler.Style.Defs.run/2","doc":"","ref":"Styler.Style.Defs.html#run/2"},{"type":"module","title":"Styler.Style.Deprecations","doc":"Transformations to soft or hard deprecations introduced on newer Elixir releases","ref":"Styler.Style.Deprecations.html"},{"type":"function","title":"Styler.Style.Deprecations.run/2","doc":"","ref":"Styler.Style.Deprecations.html#run/2"},{"type":"module","title":"Styler.Style.ModuleDirectives","doc":"Styles up module directives!\n\nThis Style will expand multi-aliases/requires/imports/use and sort the directive within its groups (except `use`s, which cannot be sorted)\nIt also adds a blank line after each directive group.","ref":"Styler.Style.ModuleDirectives.html"},{"type":"module","title":"Credo rules - Styler.Style.ModuleDirectives","doc":"Rewrites for the following Credo rules:\n\n  * `Credo.Check.Consistency.MultiAliasImportRequireUse` (force expansion)\n  * `Credo.Check.Readability.AliasOrder` (we sort `__MODULE__`, which credo doesn't)\n  * `Credo.Check.Readability.ModuleDoc` (adds `@moduledoc false` if missing. includes `*.exs` files)\n  * `Credo.Check.Readability.MultiAlias`\n  * `Credo.Check.Readability.StrictModuleLayout` (see section below for details)\n  * `Credo.Check.Readability.UnnecessaryAliasExpansion`","ref":"Styler.Style.ModuleDirectives.html#module-credo-rules"},{"type":"module","title":"Strict Layout - Styler.Style.ModuleDirectives","doc":"**This can break your code.**\n\nModules directives are sorted into the following order:\n\n  * `@shortdoc`\n  * `@moduledoc`\n  * `@behaviour`\n  * `use`\n  * `import`\n  * `alias`\n  * `require`\n  * everything else (unchanged)\n\nIf any of the sorted directives had a dependency on code that is now below it, your code will fail to compile after being styled.\n\nFor instance, the following will be broken because the module attribute definition will\nbe moved below the `use` clause, meaning `@pi` is undefined when invoked.\n\n  ```elixir\n  # before\n  defmodule Approximation do\n    @pi 3.14\n    use Math, pi: @pi\n  end\n\n  # after\n  defmodule Approximation do\n    @moduledoc false\n    use Math, pi: @pi\n    @pi 3.14\n  end\n  ```\n\nFor now, it's up to you to come up with a fix for this issue. Sorry!","ref":"Styler.Style.ModuleDirectives.html#module-strict-layout"},{"type":"function","title":"Styler.Style.ModuleDirectives.run/2","doc":"","ref":"Styler.Style.ModuleDirectives.html#run/2"},{"type":"module","title":"Styler.Style.Pipes","doc":"Styles pipes! In particular, don't make pipe chains of only one pipe, and some persnickety pipe chain start stuff.\n\nRewrites for the following Credo rules:\n\n  * Credo.Check.Readability.BlockPipe\n  * Credo.Check.Readability.OneArityFunctionInPipe\n  * Credo.Check.Readability.PipeIntoAnonymousFunctions\n  * Credo.Check.Readability.SinglePipe\n  * Credo.Check.Refactor.FilterCount\n  * Credo.Check.Refactor.MapInto\n  * Credo.Check.Refactor.MapJoin\n  * Credo.Check.Refactor.PipeChainStart, excluded_functions: [\"from\"]","ref":"Styler.Style.Pipes.html"},{"type":"function","title":"Styler.Style.Pipes.run/2","doc":"","ref":"Styler.Style.Pipes.html#run/2"},{"type":"module","title":"Styler.Style.SingleNode","doc":"Simple 1-1 rewrites all crammed into one module to make for more efficient traversals\n\nCredo Rules addressed:\n\n* Credo.Check.Consistency.ParameterPatternMatching\n* Credo.Check.Readability.LargeNumbers\n* Credo.Check.Readability.ParenthesesOnZeroArityDefs\n* Credo.Check.Readability.PreferImplicitTry\n* Credo.Check.Readability.WithSingleClause\n* Credo.Check.Refactor.CaseTrivialMatches\n* Credo.Check.Refactor.CondStatements\n* Credo.Check.Refactor.RedundantWithClauseResult\n* Credo.Check.Refactor.WithClauses","ref":"Styler.Style.SingleNode.html"},{"type":"function","title":"Styler.Style.SingleNode.run/2","doc":"","ref":"Styler.Style.SingleNode.html#run/2"},{"type":"module","title":"Styler.Zipper","doc":"Implements a Zipper for the Elixir AST based on Gérard Huet [Functional pearl: the\nzipper](https://www.st.cs.uni-saarland.de/edu/seminare/2005/advanced-fp/docs/huet-zipper.pdf) paper and\nClojure's `clojure.zip` API.\n\nA zipper is a data structure that represents a location in a tree from the\nperspective of the current node, also called *focus*. It is represented by a\n2-tuple where the first element is the focus and the second element is the\nmetadata/context. The metadata is `nil` when the focus is the topmost node","ref":"Styler.Zipper.html"},{"type":"function","title":"Styler.Zipper.append_child/2","doc":"Inserts the item as the rightmost child of the node at this zipper,\nwithout moving.","ref":"Styler.Zipper.html#append_child/2"},{"type":"function","title":"Styler.Zipper.children/1","doc":"Returns a list of children of the node.","ref":"Styler.Zipper.html#children/1"},{"type":"function","title":"Styler.Zipper.down/1","doc":"Returns the zipper of the leftmost child of the node at this zipper, or\nnil if no there's no children.","ref":"Styler.Zipper.html#down/1"},{"type":"function","title":"Styler.Zipper.find/3","doc":"Returns a zipper to the node that satisfies the predicate function, or `nil`\nif none is found.\n\nThe optional second parameters specifies the `direction`, defaults to\n`:next`.","ref":"Styler.Zipper.html#find/3"},{"type":"function","title":"Styler.Zipper.insert_child/2","doc":"Inserts the item as the leftmost child of the node at this zipper,\nwithout moving.","ref":"Styler.Zipper.html#insert_child/2"},{"type":"function","title":"Styler.Zipper.insert_left/2","doc":"Inserts the item as the left sibling of the node at this zipper, without\nmoving. Raises an `ArgumentError` when attempting to insert a sibling at the\ntop level.","ref":"Styler.Zipper.html#insert_left/2"},{"type":"function","title":"Styler.Zipper.insert_right/2","doc":"Inserts the item as the right sibling of the node at this zipper, without\nmoving. Raises an `ArgumentError` when attempting to insert a sibling at the\ntop level.","ref":"Styler.Zipper.html#insert_right/2"},{"type":"function","title":"Styler.Zipper.insert_siblings/2","doc":"Inserts many siblings to the right.\n\nEquivalent to\n\n    Enum.reduce(siblings, zipper, &Zipper.insert_right(&2, &1))","ref":"Styler.Zipper.html#insert_siblings/2"},{"type":"function","title":"Styler.Zipper.left/1","doc":"Returns the zipper of the left sibling of the node at this zipper, or nil.","ref":"Styler.Zipper.html#left/1"},{"type":"function","title":"Styler.Zipper.leftmost/1","doc":"Returns the leftmost sibling of the node at this zipper, or itself.","ref":"Styler.Zipper.html#leftmost/1"},{"type":"function","title":"Styler.Zipper.next/1","doc":"Returns the following zipper in depth-first pre-order.","ref":"Styler.Zipper.html#next/1"},{"type":"function","title":"Styler.Zipper.node/1","doc":"Returns the node at the zipper.","ref":"Styler.Zipper.html#node/1"},{"type":"function","title":"Styler.Zipper.prepend_siblings/2","doc":"Inserts many siblings to the left.\n\nEquivalent to\n\n    Enum.reduce(siblings, zipper, &Zipper.insert_left(&2, &1))","ref":"Styler.Zipper.html#prepend_siblings/2"},{"type":"function","title":"Styler.Zipper.prev/1","doc":"Returns the previous zipper in depth-first pre-order. If it's already at\nthe end, it returns nil.","ref":"Styler.Zipper.html#prev/1"},{"type":"function","title":"Styler.Zipper.remove/1","doc":"Removes the node at the zipper, returning the zipper that would have preceded\nit in a depth-first walk.","ref":"Styler.Zipper.html#remove/1"},{"type":"function","title":"Styler.Zipper.replace/2","doc":"Replaces the current node in the zipper with a new node.","ref":"Styler.Zipper.html#replace/2"},{"type":"function","title":"Styler.Zipper.replace_children/2","doc":"Returns a new node, given an existing node and new children.","ref":"Styler.Zipper.html#replace_children/2"},{"type":"function","title":"Styler.Zipper.right/1","doc":"Returns the zipper of the right sibling of the node at this zipper, or nil.","ref":"Styler.Zipper.html#right/1"},{"type":"function","title":"Styler.Zipper.rightmost/1","doc":"Returns the rightmost sibling of the node at this zipper, or itself.","ref":"Styler.Zipper.html#rightmost/1"},{"type":"function","title":"Styler.Zipper.root/1","doc":"Walks the zipper all the way up and returns the root node.","ref":"Styler.Zipper.html#root/1"},{"type":"function","title":"Styler.Zipper.skip/2","doc":"Returns the zipper of the right sibling of the node at this zipper, or the\nnext zipper when no right sibling is available.\n\nThis allows to skip subtrees while traversing the siblings of a node.\n\nThe optional second parameters specifies the `direction`, defaults to\n`:next`.\n\nIf no right/left sibling is available, this function returns the same value as\n`next/1`/`prev/1`.\n\nThe function `skip/1` behaves like the `:skip` in `traverse_while/2` and\n`traverse_while/3`.","ref":"Styler.Zipper.html#skip/2"},{"type":"function","title":"Styler.Zipper.top/1","doc":"Walks the zipper all the way up and returns the top zipper.","ref":"Styler.Zipper.html#top/1"},{"type":"function","title":"Styler.Zipper.traverse/2","doc":"Traverses the tree in depth-first pre-order calling the given function for\neach node.\n\nIf the zipper is not at the top, just the subtree will be traversed.\n\nThe function must return a zipper.","ref":"Styler.Zipper.html#traverse/2"},{"type":"function","title":"Styler.Zipper.traverse/3","doc":"Traverses the tree in depth-first pre-order calling the given function for\neach node with an accumulator.\n\nIf the zipper is not at the top, just the subtree will be traversed.","ref":"Styler.Zipper.html#traverse/3"},{"type":"function","title":"Styler.Zipper.traverse_while/2","doc":"Traverses the tree in depth-first pre-order calling the given function for\neach node.\n\nThe traversing will continue if the function returns `{:cont, zipper}`,\nskipped for `{:skip, zipper}` and halted for `{:halt, zipper}`\n\nIf the zipper is not at the top, just the subtree will be traversed.\n\nThe function must return a zipper.","ref":"Styler.Zipper.html#traverse_while/2"},{"type":"function","title":"Styler.Zipper.traverse_while/3","doc":"Traverses the tree in depth-first pre-order calling the given function for\neach node with an accumulator.\n\nThe traversing will continue if the function returns `{:cont, zipper, acc}`,\nskipped for `{:skip, zipper, acc}` and halted for `{:halt, zipper, acc}`\n\nIf the zipper is not at the top, just the subtree will be traversed.","ref":"Styler.Zipper.html#traverse_while/3"},{"type":"function","title":"Styler.Zipper.up/1","doc":"Returns the zipper of the parent of the node at this zipper, or nil if at the\ntop.","ref":"Styler.Zipper.html#up/1"},{"type":"function","title":"Styler.Zipper.update/2","doc":"Replaces the current node in the zipper with the result of applying `fun` to\nthe node.","ref":"Styler.Zipper.html#update/2"},{"type":"function","title":"Styler.Zipper.zip/1","doc":"Creates a zipper from a tree node.","ref":"Styler.Zipper.html#zip/1"},{"type":"type","title":"Styler.Zipper.command/0","doc":"","ref":"Styler.Zipper.html#t:command/0"},{"type":"opaque","title":"Styler.Zipper.path/0","doc":"","ref":"Styler.Zipper.html#t:path/0"},{"type":"type","title":"Styler.Zipper.t/0","doc":"","ref":"Styler.Zipper.html#t:t/0"},{"type":"type","title":"Styler.Zipper.tree/0","doc":"","ref":"Styler.Zipper.html#t:tree/0"},{"type":"type","title":"Styler.Zipper.zipper/0","doc":"","ref":"Styler.Zipper.html#t:zipper/0"},{"type":"exception","title":"Styler.StyleError","doc":"Wraps errors raised by Styles during tree traversal.","ref":"Styler.StyleError.html"},{"type":"function","title":"Styler.StyleError.message/1","doc":"","ref":"Styler.StyleError.html#message/1"},{"type":"extras","title":"Changelog","doc":"# Changelog","ref":"changelog.html"},{"type":"extras","title":"main - Changelog","doc":"","ref":"changelog.html#main"},{"type":"extras","title":"v0.11.4 - Changelog","doc":"#","ref":"changelog.html#v0-11-4"},{"type":"extras","title":"Improvements - Changelog","doc":"Shoutout @milmazz for all the deprecation work below =)\n\n* Deprecations: Rewrite 1.16 Deprecations (h/t @milmazz for all the work here)\n  * add `//1` step to `Enum.slice/2|String.slice/2` with decreasing ranges\n  * `File.stream!(file, options, line_or_bytes)` => `File.stream!(file, line_or_bytes, options)`\n* Deprecations `Path.safe_relative_to/2` => `Path.safe_relative/2`","ref":"changelog.html#improvements"},{"type":"extras","title":"v0.11.3 - Changelog","doc":"#","ref":"changelog.html#v0-11-3"},{"type":"extras","title":"Fixes - Changelog","doc":"* directives: fix infinite loop when encountering `@spec import(...) :: ...` (Closes #115, h/t @kerryb)\n* `with`: fix deletion of arrow-less `with` statements within function invocations","ref":"changelog.html#fixes"},{"type":"extras","title":"v0.11.2 - Changelog","doc":"#","ref":"changelog.html#v0-11-2"},{"type":"extras","title":"Fixes - Changelog","doc":"* `pipes`: fix unpiping do-blocks into variables when the parent expression is a function invocation\n    like `a(if x do y end |> z(), b)` (Closes #114, h/t @wkirschbaum)","ref":"changelog.html#fixes"},{"type":"extras","title":"v0.11.1 - Changelog","doc":"#","ref":"changelog.html#v0-11-1"},{"type":"extras","title":"Fixes - Changelog","doc":"* `with`: fix `with` replacement when it's the only child of a `do` or `->` block (Closes #107, h/t @kerryb -- turns out those edge cases _did_ exist in the wild!)","ref":"changelog.html#fixes"},{"type":"extras","title":"v0.11.0 - Changelog","doc":"#","ref":"changelog.html#v0-11-0"},{"type":"extras","title":"Improvements - Changelog","doc":"##","ref":"changelog.html#improvements"},{"type":"extras","title":"Comments - Changelog","doc":"Styler will no longer make comments jump around in any situation, and will move comments with the appropriate node in all cases but module directive rearrangement (where they'll just be left behind - sorry! we're still working on it).\n\n* Keep comments in logical places when rewriting if/unless/cond/with (#79, #97, #101, #103)\n\n##","ref":"changelog.html#comments"},{"type":"extras","title":"With Statements - Changelog","doc":"This release has a slew of improvements for `with` statements. It's not surprising that there's lots of style rules for `with` given that just about any `case`, `if`, or even `cond do` could also be expressed as a `with`. They're very powerful! And with great power...\n\n* style trivial pattern matches ala `lhs <- rhs` to `lhs = rhs` (#86)\n* style `_ <- rhs` to `rhs`\n* style keyword `, do: ` to `do end` rather than wrapping multiple statements in parens\n* style statements all the way to `if` statements when appropriate (#100)\n\n##","ref":"changelog.html#with-statements"},{"type":"extras","title":"Other - Changelog","doc":"* Rewrite `{Map|Keyword}.merge(single_key: value)` to use `put/3` instead (#96)\n\n#","ref":"changelog.html#other"},{"type":"extras","title":"Fixes - Changelog","doc":"* `with`: various edge cases we can only hope no one's encountered and thus never reported","ref":"changelog.html#fixes"},{"type":"extras","title":"v0.10.5 - Changelog","doc":"After being bitten by two of them in a row, Styler's test suite now makes sure that there are no\nidempotency bugs as part of its tests.\n\nIn short, we now have `assert style(x) == style(style(x))` as part of every test. Sorry for not thinking to include this before :)\n\n#","ref":"changelog.html#v0-10-5"},{"type":"extras","title":"Fixes - Changelog","doc":"* alias: fix single-module alias deletion newlines bug\n* comments: ensure all generated nodes always include line meta (#101)","ref":"changelog.html#fixes"},{"type":"extras","title":"v0.10.4 - Changelog","doc":"#","ref":"changelog.html#v0-10-4"},{"type":"extras","title":"Improvements - Changelog","doc":"* alias: delete noop single-module aliases (`alias Foo`, #87, h/t @mgieger)\n\n#","ref":"changelog.html#improvements"},{"type":"extras","title":"Fixes - Changelog","doc":"* pipes: unnest all pipe starts in one pass (`f(g(h(x))) |> j()` => `x |> h() |> g() |> f() |> j()`, #94, h/t @tomjschuster)","ref":"changelog.html#fixes"},{"type":"extras","title":"v0.10.3 - Changelog","doc":"#","ref":"changelog.html#v0-10-3"},{"type":"extras","title":"Improvements - Changelog","doc":"* charlists: leave charlist rewriting to elixir's formatter on elixir >= 1.15\n\n#","ref":"changelog.html#improvements"},{"type":"extras","title":"Fixes - Changelog","doc":"* charlists: rewrite empty charlist to use sigil (`''` => `~c\"\"`)\n* pipes: don't blow up extracting fully-qualified macros (`Foo.bar do end |> foo()`, #91, h/t @NikitaNaumenko)","ref":"changelog.html#fixes"},{"type":"extras","title":"v0.10.2 - Changelog","doc":"#","ref":"changelog.html#v0-10-2"},{"type":"extras","title":"Improvements - Changelog","doc":"* `with`: remove identity singleton else clause (eg `else {:error, e} -> {:error, e} end`, `else error -> error end`)","ref":"changelog.html#improvements"},{"type":"extras","title":"v0.10.1 - Changelog","doc":"#","ref":"changelog.html#v0-10-1"},{"type":"extras","title":"Fixes - Changelog","doc":"* Fix function head shrink-failures causing comments to jump into blocks (Closes #67, h/t @APB9785)","ref":"changelog.html#fixes"},{"type":"extras","title":"v0.10.0 - Changelog","doc":"#","ref":"changelog.html#v0-10-0"},{"type":"extras","title":"Improvements - Changelog","doc":"* hoist all block-starts to pipes to their own variables (makes styler play better with piped macros)\n\n#","ref":"changelog.html#improvements"},{"type":"extras","title":"Fixes - Changelog","doc":"* fix pipes starting with a macro do-block creating invalid ast (#83, h/t @mhanberg)","ref":"changelog.html#fixes"},{"type":"extras","title":"v0.9.7 - Changelog","doc":"#","ref":"changelog.html#v0-9-7"},{"type":"extras","title":"Fixes - Changelog","doc":"* rewrite pipes starting with `quote` blocks like we do with `case|if|cond|with` blocks (#82, h/t @SteffenDE)","ref":"changelog.html#fixes"},{"type":"extras","title":"v0.9.6 - Changelog","doc":"#","ref":"changelog.html#v0-9-6"},{"type":"extras","title":"Breaking Change - Changelog","doc":"* removed `mix style` task","ref":"changelog.html#breaking-change"},{"type":"extras","title":"v0.9.5 - Changelog","doc":"#","ref":"changelog.html#v0-9-5"},{"type":"extras","title":"Fixes - Changelog","doc":"* fix mistaking `Timex.now/1` in a pipe for `Timex.now/0` (#66, h/t @sabiwara)\n\n#","ref":"changelog.html#fixes"},{"type":"extras","title":"Removed style - Changelog","doc":"* stop rewriting `Timex.today/0` given that we allow `Timex.today/1` -- too inconsistent.","ref":"changelog.html#removed-style"},{"type":"extras","title":"v0.9.4 - Changelog","doc":"#","ref":"changelog.html#v0-9-4"},{"type":"extras","title":"Improvements - Changelog","doc":"* `if` statements: drop `else` clauses whose body is simply `nil`","ref":"changelog.html#improvements"},{"type":"extras","title":"v0.9.3 - Changelog","doc":"#","ref":"changelog.html#v0-9-3"},{"type":"extras","title":"Fixes - Changelog","doc":"* fix `unless a do b else c end` rewrites to `if` not flopping do/else bodies! (#77, h/t @jcowgar)\n* fix pipes styling ranges with steps (`a..b//c`) incorrectly (#76, h/t @cschmatzler)","ref":"changelog.html#fixes"},{"type":"extras","title":"v0.9.2 - Changelog","doc":"#","ref":"changelog.html#v0-9-2"},{"type":"extras","title":"Fixes - Changelog","doc":"* fix exception styling module attributes named `@def` (we confused them with real `def`s, whoops!) (#75, h/t @randycoulman)","ref":"changelog.html#fixes"},{"type":"extras","title":"v0.9.1 - Changelog","doc":"the boolean blocks edition!\n\n#","ref":"changelog.html#v0-9-1"},{"type":"extras","title":"Improvements - Changelog","doc":"* auto-fix `Credo.Check.Refactor.CondStatements` (detects any truthy atom, not just `true`)\n* if/unless rewrites:\n  - `Credo.Check.Refactor.NegatedConditionsWithElse`\n  - `Credo.Check.Refactor.NegatedConditionsInUnless`\n  - `Credo.Check.Refactor.UnlessWithElse`","ref":"changelog.html#improvements"},{"type":"extras","title":"v0.9.0 - Changelog","doc":"the with statement edition!\n\n#","ref":"changelog.html#v0-9-0"},{"type":"extras","title":"Improvements - Changelog","doc":"* Added right-hand-pattern-matching rewrites to `for` and `with` left arrow expressions `<-`\n  (ex: `with map = %{} <- foo()` => `with %{} = map <- foo`)\n* `with` statement rewrites, solving the following credo rules\n  * `Credo.Check.Readability.WithSingleClause`\n  * `Credo.Check.Refactor.RedundantWithClauseResult`\n  * `Credo.Check.Refactor.WithClauses`","ref":"changelog.html#improvements"},{"type":"extras","title":"v0.8.5 - Changelog","doc":"#","ref":"changelog.html#v0-8-5"},{"type":"extras","title":"Fixes - Changelog","doc":"* Fixed exception when encountering non-arrowed case statements ala `case foo, do: unquote(quoted)` (#69, h/t @brettinternet, nice)","ref":"changelog.html#fixes"},{"type":"extras","title":"v0.8.4 - Changelog","doc":"#","ref":"changelog.html#v0-8-4"},{"type":"extras","title":"Fixes - Changelog","doc":"* Timex related fixes (#66):\n  * Rewrite `Timex.now/1` to `DateTime.now!/1` instead of `DateTime.utc_now/1`\n  * Only rewrite `Timex.today/0`, don't change `Timex.today/1`","ref":"changelog.html#fixes"},{"type":"extras","title":"v0.8.3 - Changelog","doc":"#","ref":"changelog.html#v0-8-3"},{"type":"extras","title":"Improvements - Changelog","doc":"* DateTime rewrites (#62, ht @milmazz)\n  * `DateTime.compare` => `DateTime.{before/after}` (elixir >= 1.15)\n  * `Timex.now` => `DateTime.utc_now`\n  * `Timex.today` => `Date.utc_today`\n\n#","ref":"changelog.html#improvements"},{"type":"extras","title":"Fixes - Changelog","doc":"* Pipes: add  `!=`, `!==`, `===`, `and`, and `or` to list of valid infix operators (#64)","ref":"changelog.html#fixes"},{"type":"extras","title":"v0.8.2 - Changelog","doc":"#","ref":"changelog.html#v0-8-2"},{"type":"extras","title":"Fixes - Changelog","doc":"* Pipes always de-sugars keyword lists when unpiping them (#60)","ref":"changelog.html#fixes"},{"type":"extras","title":"v0.8.1 - Changelog","doc":"#","ref":"changelog.html#v0-8-1"},{"type":"extras","title":"Fixes - Changelog","doc":"* ModuleDirectives doesn't mistake variables for directives (#57, h/t @leandrocp)","ref":"changelog.html#fixes"},{"type":"extras","title":"v0.8.0 - Changelog","doc":"#","ref":"changelog.html#v0-8-0"},{"type":"extras","title":"Improvements (Bug Fix!?) - Changelog","doc":"* ModuleDirectives no longer throws comments around a file when hoisting directives up (#53)","ref":"changelog.html#improvements-bug-fix"},{"type":"extras","title":"v0.7.14 - Changelog","doc":"#","ref":"changelog.html#v0-7-14"},{"type":"extras","title":"Improvements - Changelog","doc":"* rewrite `Logger.warn/1,2` to `Logger.warning/1,2` due to Elixir 1.15 deprecation","ref":"changelog.html#improvements"},{"type":"extras","title":"v0.7.13 - Changelog","doc":"#","ref":"changelog.html#v0-7-13"},{"type":"extras","title":"Fixes - Changelog","doc":"* don't unpipe single-piped `unquote` expressions (h/t @elliottneilclark)","ref":"changelog.html#fixes"},{"type":"extras","title":"v0.7.12 - Changelog","doc":"#","ref":"changelog.html#v0-7-12"},{"type":"extras","title":"Fixes - Changelog","doc":"* fix 0-arity paren removal on metaprogramming creating uncompilable code (h/t @simonprev)","ref":"changelog.html#fixes"},{"type":"extras","title":"v0.7.11 - Changelog","doc":"#","ref":"changelog.html#v0-7-11"},{"type":"extras","title":"Fixes - Changelog","doc":"* fix crash from `mix style` running plugins as part of formatting (no longer runs formatter plugins)\n\n#","ref":"changelog.html#fixes"},{"type":"extras","title":"Improvements - Changelog","doc":"* single-quote charlists are rewritten to use the `~c` sigil (`'foo'` -> `~c'foo'`) (h/t @fhunleth)\n* `mix style` warns the user that Styler is primarily meant to be used as a plugin","ref":"changelog.html#improvements"},{"type":"extras","title":"v0.7.10 - Changelog","doc":"#","ref":"changelog.html#v0-7-10"},{"type":"extras","title":"Fixes - Changelog","doc":"* fix crash when encountering single-quote charlists (h/t @fhunleth)\n\n#","ref":"changelog.html#fixes"},{"type":"extras","title":"Improvements - Changelog","doc":"* single-quote charlists are rewritten to use the `~c` sigil (`'foo'` -> `~c'foo'`)\n* when encountering `_ = bar ->`, replace it with `bar ->`","ref":"changelog.html#improvements"},{"type":"extras","title":"v0.7.9 - Changelog","doc":"#","ref":"changelog.html#v0-7-9"},{"type":"extras","title":"Fixes - Changelog","doc":"* Fix a toggle state resulting from (ahem, nonsense) code like `_ = bar ->` encountering ParameterPatternMatching style","ref":"changelog.html#fixes"},{"type":"extras","title":"v0.7.8 - Changelog","doc":"#","ref":"changelog.html#v0-7-8"},{"type":"extras","title":"Fixes - Changelog","doc":"* Fix crash trying to remove 0-arity parens from metaprogramming ala `def unquote(foo)()`","ref":"changelog.html#fixes"},{"type":"extras","title":"v0.7.7 - Changelog","doc":"#","ref":"changelog.html#v0-7-7"},{"type":"extras","title":"Improvements - Changelog","doc":"* Rewrite `Enum.into/2,3` into `Map.new/1,2` when the collectable is `%{}` or `Map.new/0`","ref":"changelog.html#improvements"},{"type":"extras","title":"v0.7.6 - Changelog","doc":"#","ref":"changelog.html#v0-7-6"},{"type":"extras","title":"Fixes - Changelog","doc":"* Fix crash when single pipe had inner defs (h/t [@michallepicki](https://github.com/adobe/elixir-styler/issues/39))","ref":"changelog.html#fixes"},{"type":"extras","title":"v0.7.5 - Changelog","doc":"#","ref":"changelog.html#v0-7-5"},{"type":"extras","title":"Fixes - Changelog","doc":"* Fix bug from `ParameterPatternMatching` implementation that re-ordered pattern matching in `cond do` `->` clauses","ref":"changelog.html#fixes"},{"type":"extras","title":"v0.7.4 - Changelog","doc":"#","ref":"changelog.html#v0-7-4"},{"type":"extras","title":"Features - Changelog","doc":"* Implement `Credo.Check.Readability.PreferImplicitTry`\n* Implement `Credo.Check.Consistency.ParameterPatternMatching` for `def|defp|fn|case`","ref":"changelog.html#features"},{"type":"extras","title":"v0.7.3 - Changelog","doc":"#","ref":"changelog.html#v0-7-3"},{"type":"extras","title":"Features - Changelog","doc":"* Remove parens from 0-arity function definitions (`Credo.Check.Readability.ParenthesesOnZeroArityDefs`)","ref":"changelog.html#features"},{"type":"extras","title":"v0.7.2 - Changelog","doc":"#","ref":"changelog.html#v0-7-2"},{"type":"extras","title":"Features - Changelog","doc":"* Rewrite `case ... true -> ...; _ -> ...` to `if` statements as well","ref":"changelog.html#features"},{"type":"extras","title":"v0.7.1 - Changelog","doc":"#","ref":"changelog.html#v0-7-1"},{"type":"extras","title":"Features - Changelog","doc":"* Rewrite `case ... true / else ->` to be `if` statements","ref":"changelog.html#features"},{"type":"extras","title":"v0.7.0 - Changelog","doc":"#","ref":"changelog.html#v0-7-0"},{"type":"extras","title":"Features - Changelog","doc":"* `Styler.Style.Simple`:\n    * Optimize `Enum.reverse(foo) ++ bar` to `Enum.reverse(foo, bar)`\n* `Styler.Style.Pipes`:\n    * Rewrite `|> (& ...).()` to `|> then(& ...)` (`Credo.Check.Readability.PipeIntoAnonymousFunctions`)\n    * Add parens to 1-arity pipe functions (`Credo.Check.Readability.OneArityFunctionInPipe`)\n    * Optimize `a |> Enum.reverse() |> Enum.concat(enum)` to `Enum.reverse(a, enum)`","ref":"changelog.html#features"},{"type":"extras","title":"v0.6.1 - Changelog","doc":"#","ref":"changelog.html#v0-6-1"},{"type":"extras","title":"Improvements - Changelog","doc":"* Better error handling: `mix format` will still format files if a style fails\n\n#","ref":"changelog.html#improvements"},{"type":"extras","title":"Fixes - Changelog","doc":"* `mix style`: only run on `.ex` and `.exs` files\n* `ModuleDirectives`: now expands `alias __MODULE__.{A, B}` (h/t [@adriankumpf](https://github.com/adriankumpf))","ref":"changelog.html#fixes"},{"type":"extras","title":"v0.6.0 - Changelog","doc":"#","ref":"changelog.html#v0-6-0"},{"type":"extras","title":"Features - Changelog","doc":"* `mix style`: brought back to life for folks who want to incrementally introduce Styler\n\n#","ref":"changelog.html#features"},{"type":"extras","title":"Fixes - Changelog","doc":"* `Styler.Style.Pipes`:\n   * include `x in y` and `^foo` (for ecto) as a valid pipe starts\n   * work even harder to keep rewrites on one line","ref":"changelog.html#fixes"},{"type":"extras","title":"v0.5.2 - Changelog","doc":"#","ref":"changelog.html#v0-5-2"},{"type":"extras","title":"Fixes - Changelog","doc":"* `ModuleDirectives`: handle dynamic module names\n* `Pipes`: include `Ecto.Query.from` and `Query.from` as valid pipe starts","ref":"changelog.html#fixes"},{"type":"extras","title":"v0.5.1 - Changelog","doc":"#","ref":"changelog.html#v0-5-1"},{"type":"extras","title":"Improvements - Changelog","doc":"* Sped up styling just a little bit","ref":"changelog.html#improvements"},{"type":"extras","title":"v0.5.0 - Changelog","doc":"#","ref":"changelog.html#v0-5-0"},{"type":"extras","title":"Improvements - Changelog","doc":"* `Styler` now implements `Mix.Task.Format`, meaning it is now an Elixir formatter plugin.\nSee the README for new installation & usage instructions\n\n#","ref":"changelog.html#improvements"},{"type":"extras","title":"Breaking Change! Wooo! - Changelog","doc":"* the `mix style` task has been removed","ref":"changelog.html#breaking-change-wooo"},{"type":"extras","title":"v0.4.1 - Changelog","doc":"#","ref":"changelog.html#v0-4-1"},{"type":"extras","title":"Improvements - Changelog","doc":"* `Pipes` rewrites `|> Enum.into(%{}[, mapper])` and `Enum.into(Map.new()[, mapper])` to `Map.new/1,2` calls","ref":"changelog.html#improvements"},{"type":"extras","title":"v0.4.0 - Changelog","doc":"#","ref":"changelog.html#v0-4-0"},{"type":"extras","title":"Improvements - Changelog","doc":"* `Pipes` rewrites some two-step processes into one, fixing these credo issues in pipe chains:\n    * `Credo.Check.Refactor.FilterCount`\n    * `Credo.Check.Refactor.MapJoin`\n    * `Credo.Check.Refactor.MapInto`\n\n#","ref":"changelog.html#improvements"},{"type":"extras","title":"Fixes - Changelog","doc":"* `ModuleDirectives` handles even weirder places to hide your aliases (anonymous functions, in this case)\n* `Pipes` tries even harder to keep single-pipe rewrites of invocations on one line","ref":"changelog.html#fixes"},{"type":"extras","title":"v0.3.1 - Changelog","doc":"#","ref":"changelog.html#v0-3-1"},{"type":"extras","title":"Fixes - Changelog","doc":"* `Pipes`\n    * fixed omission of `==` as a valid pipe start operator (h/t @peake100 for the issue)\n    * fixed rewrite of `a |> b`, where `b` was invoked without parenthesis","ref":"changelog.html#fixes"},{"type":"extras","title":"v0.3.0 - Changelog","doc":"#","ref":"changelog.html#v0-3-0"},{"type":"extras","title":"Improvements - Changelog","doc":"* Enabled `Defs` style and overhauled it to properly handles comments\n* Optimized and tweaked `ModuleDirectives` style\n    * Now culls newlines between \"groups\" of the same directive\n    * sorts `@behaviour` directives\n    * orders directives within non defmodule contexts (eg, a `def do`) if there's at least one `alias|require|use|import`\n\n#","ref":"changelog.html#improvements"},{"type":"extras","title":"Fixes - Changelog","doc":"* `Pipes` will try to keep single-pipe rewrites on one line","ref":"changelog.html#fixes"},{"type":"extras","title":"v0.2.0 - Changelog","doc":"#","ref":"changelog.html#v0-2-0"},{"type":"extras","title":"Improvements - Changelog","doc":"* Added `ModuleDirectives` style\n    * Note that this is potentially destructive in some rare cases. See moduledoc for more.\n    * This supersedes the `Aliases` style, which has been removed.\n* `mix style -` reads and writes to stdin/stdout\n\n#","ref":"changelog.html#improvements"},{"type":"extras","title":"Fixes - Changelog","doc":"* `Pipes` style is now aware of `unless` blocks","ref":"changelog.html#fixes"},{"type":"extras","title":"v0.1.1 - Changelog","doc":"#","ref":"changelog.html#v0-1-1"},{"type":"extras","title":"Improvements - Changelog","doc":"* Lots of README tweaking =)\n* Optimized some Zipper operations\n* Added `Simple` style, replacing the following Credo rule:\n    * `Credo.Check.Readability.LargeNumbers`\n\n#","ref":"changelog.html#improvements"},{"type":"extras","title":"Fixes - Changelog","doc":"* Exceptions while parsing code now appropriately render filename rather than `nofile:xx`\n* Fixed opaque `Zipper.path()` typespec implementation mismatches (thanks @sega-yarkin)\n* Made `ex_doc` dev only, removing it as a dependency for users of Styler","ref":"changelog.html#fixes"},{"type":"extras","title":"v0.1.0 - Changelog","doc":"#","ref":"changelog.html#v0-1-0"},{"type":"extras","title":"Improvements - Changelog","doc":"* Initial release of Styler\n* Added `Aliases` style, replacing the following Credo rules:\n    * `Credo.Check.Readability.AliasOrder`\n    * `Credo.Check.Readability.MultiAlias`\n    * `Credo.Check.Readability.UnnecessaryAliasExpansion`\n* Added `Pipes` style, replacing the following Credo rules:\n    * `Credo.Check.Readability.BlockPipe`\n    * `Credo.Check.Readability.SinglePipe`\n    * `Credo.Check.Refactor.PipeChainStart`\n* Added `Defs` style (currently disabled by default)","ref":"changelog.html#improvements"},{"type":"extras","title":"Styler","doc":"# Styler\n\nStyler is an Elixir formatter plugin that's combination of `mix format` and `mix credo`, except instead of telling\nyou what's wrong, it just rewrites the code for you to fit its style rules.\n\nYou can learn more about the history, purpose and implementation of Styler from our talk: [Styler: Elixir Style-Guide Enforcer @ GigCity Elixir 2023](https://www.youtube.com/watch?v=6pF8Hl5EuD4)","ref":"readme.html"},{"type":"extras","title":"Installation - Styler","doc":"Add `:styler` as a dependency to your project's `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:styler, \"~> 0.11\", only: [:dev, :test], runtime: false},\n  ]\nend\n```\n\nThen add `Styler` as a plugin to your `.formatter.exs` file\n\n```elixir\n[\n  plugins: [Styler]\n]\n```\n\nAnd that's it! Now when you run `mix format` you'll also get the benefits of Styler's *definitely-always-right* style fixes.\n\n#","ref":"readme.html#installation"},{"type":"extras","title":"Configuration - Styler","doc":"There isn't any! This is intentional.\n\nStyler is @adobe's internal Style Guide Enforcer - allowing exceptions to the styles goes against that ethos. Happily, it's open source and thus yours to do with as you will =)","ref":"readme.html#configuration"},{"type":"extras","title":"Features (or as we call them, \"Styles\") - Styler","doc":"At this point, Styler does a lot. We've catalogued a list of Credo rules that it automatically fixes, but it does some things -\nlike shrinking function heads down to a single line when possible - that Credo doesn't care about.\n\nUltimately, the best way to see what Styler does is to just try it out! What could go wrong? (You're using version control, right?)\n\n#","ref":"readme.html#features-or-as-we-call-them-styles"},{"type":"extras","title":"Credo Rules Styler Replaces - Styler","doc":"If you're using Credo and Styler, **we recommend disabling these rules in `.credo.exs`** to save on unnecessary checks in CI.\n\nDisabling the rules means updating your `.credo.exs` depending on your configuration:\n\n- if you're using `checks: %{enabled: [...]}`, ensure none of the checks are listed in your enabled checks\n- if you're using `checks: %{disabled: [...]}`, copy/paste the snippet below into the list\n- if you're using `checks: [...]`, copy/paste the snippet below into the list and ensure none of the checks appear earlier in the list\n\n```elixir\n# Styler Rewrites\n#\n# The following rules are automatically rewritten by Styler and so disabled here to save time\n# Some of the rules have `priority: :high`, meaning Credo runs them unless we explicitly disable them\n# (removing them from this file wouldn't be enough, the `false` is required)\n#\n# Some rules have a comment before them explaining ways Styler deviates from the Credo rule.\n#\n# always expands `A.{B, C}`\n{Credo.Check.Consistency.MultiAliasImportRequireUse, false},\n# including `case`, `fn` and `with` statements\n{Credo.Check.Consistency.ParameterPatternMatching, false},\n{Credo.Check.Readability.AliasOrder, false},\n{Credo.Check.Readability.BlockPipe, false},\n# goes further than formatter - fixes bad underscores, eg: `100_00` -> `10_000`\n{Credo.Check.Readability.LargeNumbers, false},\n# adds `@moduledoc false`\n{Credo.Check.Readability.ModuleDoc, false},\n{Credo.Check.Readability.MultiAlias, false},\n{Credo.Check.Readability.OneArityFunctionInPipe, false},\n# removes parens\n{Credo.Check.Readability.ParenthesesOnZeroArityDefs, false},\n{Credo.Check.Readability.PipeIntoAnonymousFunctions, false},\n{Credo.Check.Readability.PreferImplicitTry, false},\n{Credo.Check.Readability.SinglePipe, false},\n# **potentially breaks compilation** - see **Troubleshooting** section below\n{Credo.Check.Readability.StrictModuleLayout, false},\n{Credo.Check.Readability.UnnecessaryAliasExpansion, false},\n{Credo.Check.Readability.WithSingleClause, false},\n{Credo.Check.Refactor.CaseTrivialMatches, false},\n{Credo.Check.Refactor.CondStatements, false},\n# in pipes only\n{Credo.Check.Refactor.FilterCount, false},\n# in pipes only\n{Credo.Check.Refactor.MapInto, false},\n# in pipes only\n{Credo.Check.Refactor.MapJoin, false},\n{Credo.Check.Refactor.NegatedConditionsInUnless, false},\n{Credo.Check.Refactor.NegatedConditionsWithElse, false},\n# allows ecto's `from\n{Credo.Check.Refactor.PipeChainStart, false},\n{Credo.Check.Refactor.RedundantWithClauseResult, false},\n{Credo.Check.Refactor.UnlessWithElse, false},\n{Credo.Check.Refactor.WithClauses, false},\n ```","ref":"readme.html#credo-rules-styler-replaces"},{"type":"extras","title":"Your first Styling - Styler","doc":"**Speed**: Expect the first run to take some time as `Styler` rewrites violations of styles.\n\nOnce styled the first time, future styling formats shouldn't take noticeably more time.\n\n#","ref":"readme.html#your-first-styling"},{"type":"extras","title":"Troubleshooting: Compilation broke due to Module Directive rearrangement - Styler","doc":"Sometimes naively moving Module Directives around can break compilation.\n\nHere's helpers on how to manually fix that and have a happy styling for the rest of\nyour codebase's life.\n\n##","ref":"readme.html#troubleshooting-compilation-broke-due-to-module-directive-rearrangement"},{"type":"extras","title":"Alias dependency - Styler","doc":"If you have an alias that, for example, a `@behaviour` relies on, compilation will break after your first run.\nThis requires one-time manual fixing to get your repo in line with Styler's standards.\n\nFor example, if your code was this:\n```elixir\ndefmodule MyModule do\n  @moduledoc \"Implements MyBehaviour!\"\n  alias Deeply.Nested.MyBehaviour\n  @behaviour MyBehaviour\n  ...\nend\n```\n\nthen Styler will style the file like this, which cannot compile due to `MyBehaviour` not being defined.\n\n```elixir\ndefmodule MyModule do\n  @moduledoc \"Implements MyBehaviour!\"\n  @behaviour MyBehaviour  # <------ compilation error, MyBehaviour is not defined!\n\n  alias Deeply.Nested.MyBehaviour\n\n  ...\nend\n```\n\nA simple solution is to manually expand the alias with a find-replace-all like:\n`@behaviour MyBehaviour` -> `@behaviour Deeply.Nested.MyBehaviour`. It's important to specify that you only want to\nfind-replace with the `@behaviour` prefix or you'll unintentially expand `MyBehaviour` everywhere in the codebase.\n\n##","ref":"readme.html#alias-dependency"},{"type":"extras","title":"Module Attribute dependency - Styler","doc":"Another common compilation break on the first run is a `@moduledoc` that depended on another module attribute which\nwas moved below it.\n\nFor example, given the following broken code after an initial `mix format`:\n\n```elixir\ndefmodule MyGreatLibrary do\n  @moduledoc make_pretty_docs(@library_options)\n\n  @library_options [ ... ]\nend\n```\n\nYou can fix the code by moving the static value outside of the module into a naked variable and then reference it in the module.\n\nYes, this is a thing you can do in a `.ex` file =)\n\n```elixir\nlibrary_options = [ ... ]\n\ndefmodule MyGreatLibrary do\n  @moduledoc make_pretty_docs(library_options)\n\n  @library_options library_options\nend\n```","ref":"readme.html#module-attribute-dependency"},{"type":"extras","title":"Thanks & Inspiration - Styler","doc":"### [Sourceror](https://github.com/doorgan/sourceror/)\n\nThis work was inspired by earlier large-scale rewrites of an internal codebase that used the fantastic tool `Sourceror`.\n\nThe initial implementation of Styler used Sourceror, but Sourceror's AST-embedding comment algorithm slows Styler down to\nthe point that it's no longer an appropriate drop-in for `mix format`.\n\nStill, we're grateful for the inspiration Sourceror provided and the changes to the Elixir AST APIs that it drove.\n\nThe AST-Zipper implementation in this project was forked from Sourceror's implementation.\n\n### [Credo](https://github.com/rrrene/credo/)\n\nSimilarly, this project originated from one-off scripts doing large scale rewrites of an enormous codebase as part of an\neffort to enable particular Credo rules for that codebase. Credo's tests and implementations were referenced for implementing\nStyles that took the work the rest of the way. Thanks to Credo & the Elixir community at large for coalescing around\nmany of these Elixir style credos.","ref":"readme.html#thanks-inspiration"}],"content_type":"text/markdown"}